package org.bossattack.speedblazer;

import org.lwjgl.opengl.GL11;
import org.newdawn.slick.Color;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.geom.Rectangle;

public class Car {
	
	Game game;
	
	float x;
	float y;
	
	float acceleration = 0.07f;
	float friction = 0.05f;
	
	//float dx;
	float speedX = 0f;
	float speedY = 0f;
	float maxSpeed = 5.0f;
	
	float angleY = 0;
	
	public boolean upPressed = false;
	public boolean downPressed = false;
	public boolean leftPressed = false;
	public boolean rightPressed = false;
	
	Rectangle bounds;
	boolean collision = false;
	
	boolean jumping = false;
	float jumpScale = 0f;
	int jumpTime = 0, jumpTotalTime = 1000;
	
	boolean alive = true;
	int dealthDelay = 3000, deathTime;
	
	boolean ghostMode = false;
	int ghostTime, ghostDelay = 2000;
	
	int lives = 3;
	
	public Car(Game game, float x, float y) {
		this.game = game;
		
		this.x = x;
		this.y = y;
		
		bounds = new Rectangle(x, y, game.carImage.getWidth(), game.carImage.getHeight());
	}
	
	public void update(int delta) {
		
		if (ghostMode) {
			if (ghostTime < ghostDelay) {
				ghostTime += delta;
			}
			else {
				ghostMode = false;
				ghostTime = 0;
			}
		}
		
		if (collision && alive) {
			alive = false;
			game.addEntity((new BigExplosion(game, x + game.carImage.getWidth()/2f, 
												y + game.carImage.getHeight()/2f)));
			game.speed = 0;
		}
		
		if (!alive && !game.gameOver) {
			if (deathTime < dealthDelay) {
				deathTime += delta;
				return;
			}
			else {
				if (lives > 0) {
					lives -= 1;
					deathTime = 0;
					alive = true;
					collision = false;
					ghostMode = true;
					game.speed = 0.05f;
				}
				else {
					game.gameOver = true;
				}
			}
		}
		
		
		
		if (jumping) {
			jumpTime += delta;
			
			float t = jumpTime;
			
			if (t > jumpTotalTime/2f) {
				t = jumpTotalTime - jumpTime;
			}
			
			jumpScale = (t/(float)jumpTotalTime) * 1.5f;
			
			
			if (jumpTime > jumpTotalTime) {
				jumpTime = 0;
				jumpScale = 0f;
				jumping = false;
				game.speed = game.maxSpeed;
			}
		}
		
		collision = false;
		
		speedYMovement(delta);
		speedXMovement(delta);
		
		
		if (x < 75) {
			x = 75;
		}
		else if (x + game.carImage.getWidth() > 75 + 250) {
			x = 75 + 250 - game.carImage.getWidth();
		}
		
		if (y < 400) {
			y = 400;
		}
		else if (y > 450 + game.carImage.getHeight()) {
			y = 450 + game.carImage.getHeight();
			
			if (game.speed > game.minSpeed) {
				game.speed -= game.brakes * delta;
				if (game.speed < game.minSpeed) {
					game.speed = game.minSpeed;
				}
			}
		}
		
		bounds.setX(x);
		bounds.setY(y);
	}
	
	private void speedYMovement(int delta) {
		if (downPressed) {
			float dx = acceleration * delta;
			speedY += dx;
			if (speedY > maxSpeed) speedY = maxSpeed;
		}
		else if (upPressed) {
			speedY -= acceleration * delta;
			if (speedY < -maxSpeed) speedY = -maxSpeed;
		}
		
		y += speedY;
		
		if (!jumping && game.isCollision()) {
			y -= speedY*4;
			speedY = -speedY;
			game.speed = game.minSpeed;
		}
		
		if (speedY < 0) {
			speedY += friction * delta;
			if (speedY > 0) {
				speedY = 0;
			}
		}
		else if (speedY > 0) {
			speedY -= friction * delta;
			if (speedY < 0) speedY = 0;
		}
	}
	
	private void speedXMovement(int delta) {
		if (rightPressed) {
			float dx = acceleration * delta;
			speedX += dx;
			if (speedX > maxSpeed) speedX = maxSpeed;
		}
		else if (leftPressed) {
			speedX -= acceleration * delta;
			if (speedX < -maxSpeed) speedX = -maxSpeed;
		}
		
		x += speedX;
		
		if (!jumping && game.isCollision()) {
			x -= speedX*4;
			speedX = -speedX;
		}
		
		angleY = 45 * speedX/5f;
		
		if (speedX < 0) {
			speedX += friction * delta;
			if (speedX > 0) {
				speedX = 0;
			}
		}
		else if (speedX > 0) {
			speedX -= friction * delta;
			if (speedX < 0) speedX = 0;
		}
		
		
	}
	
	public void draw(Graphics g) {
		if (collision) {
			return;
		}
		
		GL11.glPushMatrix();
		
		GL11.glTranslatef(x+game.carImage.getWidth(), y, 0);
		GL11.glRotatef(angleY, 0, 1, 0);
		//GL11.glRotatef(45, 1, 0, 0);
		GL11.glTranslatef(-(x+game.carImage.getWidth()), -y, 0);
		
		
		if (jumping) {
			Image c = game.carImage.getScaledCopy(1.0f + jumpScale);
			float cx = x-c.getWidth()/2f+game.carImage.getWidth()/2f;
			float cy = y-c.getHeight()/2f+game.carImage.getHeight()/2f;
			game.carShadowImage.draw(cx + (50 * jumpScale)+3, cy + (50 * jumpScale)+3, c.getWidth(), c.getHeight(), Color.black);
			c.draw(cx, cy);
		}
		else {
			if (ghostMode) {
				game.carShadowImage.draw(x+3, y+3);
				game.carImage.draw(x, y, new Color(1, 0.5f, 0.5f, 0.5f));
			}
			else {
				game.carShadowImage.draw(x+3, y+3);
				game.carImage.draw(x, y);
			}
		}
		
		GL11.glPopMatrix();
		
		if (game.debug) {
			if(collision) {
				g.setColor(Color.red);
			}
			else {
				g.setColor(Color.yellow);
			}
			g.draw(bounds);
		}
	}
}
