package org.bossattack.speedblazer;

import java.util.ArrayList;

import org.newdawn.slick.AngelCodeFont;
import org.newdawn.slick.Color;
import org.newdawn.slick.Font;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.Music;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.Sound;
import org.newdawn.slick.SpriteSheet;
import org.newdawn.slick.state.BasicGameState;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;

public class Game extends BasicGameState {

	boolean debug = false;
	
	Image carImage;
	Image carShadowImage;
	Image civilianImage;
	Image civilianShadowImage;
	
	Image policeImage;
	Image policeLightsImage;
	
	Image rampImage;
	
	Image wallImage;
	SpriteSheet explosionSpriteSheet;
	
	Sound explosionSnd, bigExplosionSnd, rampSnd;
	
	Font smallFont;
	Font mediumFont;
	Font largeFont;
	
	float roadScroll = -80;
	
	float speed = 0.05f;
	
	float acceleration = 0.0001f;
	float brakes = 0.0005f;
	
	float maxSpeed = 0.3f;
	float minSpeed = 0.1f;
	
	Car car;
	
	ArrayList entities = new ArrayList();
	ArrayList removeList = new ArrayList();
	
	RoadLamps roadLamps;
	Speed speedWaves;
	
	// total distance travelled
	float distance = 0;
	float time = 0;
	
	Music bgm;
	
	boolean getReady = true;
	int getReadyTimer, getReadyDelay = 3000;
	String getReadyMessage = "GET READY";
	
	boolean gameOver = false;
	int gameOverTimer, gameOverDelay = 4000;
	
	public AirJunk airJunk = new AirJunk();
	
	int score = 0;
	
	LevelManager lm;
	
	public int getID() {
		return 1;
	}
	
	public void reset() {
		entities.clear();
		distance = 0;
		time = 0;
		speed = 0.05f;
		roadScroll = -80;
		
		car = new Car(this, 100, 450);
		
		speedWaves = new Speed();
		roadLamps = new RoadLamps(this);
		
		getReady = true;
		gameOver = false;
		gameOverTimer = 0;
		airJunk = new AirJunk();
		score = 0;
		
		lm = new LevelManager(this);
		
		maxSpeed = 0.3f;
	}

	public void init(GameContainer container, StateBasedGame stateBasedGame) throws SlickException {
		//enterOrtho();
		
		carImage = new Image("data/car.png");
		carShadowImage = new Image("data/carshadow.png");
		
		civilianImage = new Image("data/civilian.png");
		civilianShadowImage = new Image("data/civshadow.png");
		
		rampImage = new Image("data/ramp.png");
		
		policeImage = new Image("data/police.png");
		policeLightsImage = new Image("data/policelights.png");
		
		wallImage = new Image("data/wall.png");
		
		explosionSpriteSheet = new SpriteSheet("data/explosion.png", 64, 64);
		
		smallFont = new AngelCodeFont("data/smallfont.fnt", "data/smallfont.png");
		mediumFont = new AngelCodeFont("data/mediumfont.fnt", "data/mediumfont.png");
		largeFont = new AngelCodeFont("data/largefont.fnt", "data/largefont.png");
		
		explosionSnd = new Sound("data/exp.wav");
		bigExplosionSnd = new Sound("data/bigexp.wav");
		rampSnd = new Sound("data/ramp.wav");
		
		//bgm = new Music("data/theme2.xm");
		//bgm.loop();
		
		/*car = new Car(this, 100, 450);
		addEntity(new Wall(this, 100, 100, 100, 50));
		addEntity(new SlowRamp(this, 100, 300));
		//addEntity(new AICar(this, 200, 300));
		addEntity(new PoliceCar(this, 200, 300));
		
		speedWaves = new Speed();
		roadLamps = new RoadLamps(this);*/
	}
	
	public void addEntity(Entity entity) {
		entities.add(entity);
	}
	
	public void removeEntity(Entity entity) {
		removeList.add(entity);
	}
	
	public void update(GameContainer container, StateBasedGame stateBasedGame, int delta) throws SlickException {
		int mouseX = container.getInput().getMouseX();
		int mouseY = container.getInput().getMouseY();
		
		
		if (getReady) {
			
			if (getReadyTimer < getReadyDelay) {
				getReadyTimer += delta;
			}
			else {
				getReadyTimer = 0;
				getReady = false;
			}
			
			return;
		}
		
		if (gameOver) {
			if (gameOverTimer < gameOverDelay) {
				gameOverTimer += delta;
			}
			else {
				TitleState titleState = (TitleState) stateBasedGame.getState(0);
				titleState.setScore(score);
				stateBasedGame.enterState(0, new FadeOutTransition(Color.black, 250),
	 					 					 new FadeInTransition(Color.black, 250));
			}
			return;
		}
		
		lm.update(delta);
		
		pollInput(container.getInput(), delta);
		
		car.update(delta);
		
		roadScroll += speed * delta;
		if (roadScroll > 0) {
			roadScroll -= 80;
		}
		
		/*lampScroll += speed * delta;
		if (lampScroll > SCREENHEIGHT) {
			lampScroll = -10;
		}*/
		
		// stats
		distance += (speed * delta)/500f;
		time += delta;
		
		score = (int) (distance*10);
		
		for (int i = 0; i < entities.size(); i++) {
			Entity entity = (Entity)entities.get(i);
			entity.update(delta);
		}
		
		// collision detection
		for (int i = 0; i < entities.size(); i++) {
			Entity entity = (Entity)entities.get(i);
			
			if (entity.bounds != null) {
				if (car.bounds.intersects(entity.bounds)) {
					if (car.jumping) {
						break;
					}
					else if (entity.type == Entity.WALL) {
						if (!car.ghostMode) {
							car.collision = true;
						}
					}
					else if (entity.type == Entity.JUMPRAMP) {
						car.jumping = true;
						speed = maxSpeed +0.1f;
						rampSnd.play();
					}
					else if (entity.type == Entity.SLOWRAMP) {
						speed = minSpeed;
					}
					else if (entity.type == Entity.SPEEDRAMP) {
						speed = maxSpeed;
					}
				}
			}
		}
		
		roadLamps.update(delta);
		speedWaves.update(delta, (speed-minSpeed)/(maxSpeed-minSpeed));
		airJunk.update(delta);
		
		entities.removeAll(removeList);
		removeList.clear();
		
		if (container.getInput().isKeyPressed(Input.KEY_ESCAPE)) {
			stateBasedGame.enterState(0, new FadeOutTransition(Color.black, 250),
 					 					 new FadeInTransition(Color.black, 250));
		}
	}
	
	public boolean isCollision() {
		
		for (int i = 0; i < entities.size(); i++) {
			Entity entity = (Entity)entities.get(i);
			
			if (entity.type == Entity.AICar) {
				if (car.bounds.intersects(entity.bounds)) {
					return true;
				}
			}
		}
		
		return false;
	}
	
	private void pollInput(Input input, int delta) {
		
		if (!car.alive) return;
		
		car.leftPressed = false;
		car.rightPressed = false;
		
		car.upPressed = false;
		car.downPressed = false;
		
		if (input.isKeyDown(Input.KEY_LEFT)) {
			car.leftPressed = true;
		}
		else if (input.isKeyDown(Input.KEY_RIGHT)) {
			car.rightPressed = true;
		}
		
		if (car.jumping) return;
		
		if (input.isKeyDown(Input.KEY_UP)) {
			car.upPressed = true;
			if (speed < maxSpeed) {
				speed += acceleration * delta;
				if (speed > maxSpeed) {
					speed = maxSpeed;
				}
			}
		}
		else if (input.isKeyDown(Input.KEY_DOWN)) {
			car.downPressed = true;
			/*if (speed > minSpeed) {
				speed -= brakes * delta;
				if (speed < minSpeed) {
					speed = minSpeed;
				}
			}*/
		}
	}
	
	public void render(GameContainer container, StateBasedGame stateBasedGame, Graphics g) throws SlickException {
		g.setAntiAlias(true);
		
		g.setColor(Color.gray);
		g.fillRect(75, 0, 250, SpeedBlazer.SCREENHEIGHT);
		
		g.setColor(Color.white);
		g.translate(0, roadScroll);
		for (int i = 0; i < 9; i++) {
			g.fillRect(200-5f, i * 80, 10, 50);
			
		}
		g.resetTransform();
		
		
		
		for (int i = 0; i < entities.size(); i++) {
			Entity entity = (Entity)entities.get(i);
			entity.draw(g);
		}
		
		car.draw(g);
		
		roadLamps.draw(g);
		airJunk.draw(g);
		
		speedWaves.draw(g);
		
		//g.setColor(Color.white);
		Color fontColor = new Color(1f, 1f, 1f, 0.5f);
		
		smallFont.drawString(330, 20, "SPEED", fontColor);
		String speedString = " " + (int)(1 + 50 * (speed));
		mediumFont.drawString(350 - mediumFont.getWidth(speedString)/2f, 30, speedString, fontColor);
		//g.drawString("SPEED: ", 330, 50);
		//g.drawString(" " + (int)(1 + 50 * (speed)), 330, 70);
		
		//g.drawString("DISTANCE: ", 330, 90);
		//g.drawString(" " + (int)(distance), 330, 110);
		
		//g.drawString("TIME: ", 330, 130);
		//g.drawString(" " + (int)(time/1000f), 330, 150);
		smallFont.drawString(10, 20, "SCORE", fontColor);
		smallFont.drawString(10, 30, String.format("%010d", score), fontColor);
		
		smallFont.drawString(335, 540, "LIVES", fontColor);//Sg.drawString("LIVES: ", 330, 170);
		mediumFont.drawString(345, 550, "" + car.lives, fontColor);
		//g.drawString(" " + car.lives, 330, 190);
		
		if (getReady) {
			g.setColor(new Color(0.3f, 0.3f, 0.3f, 0.7f));
			g.fillRoundRect(85, 300, 230, 75, 10);
			mediumFont.drawString(200-mediumFont.getWidth(getReadyMessage)/2f, 300, getReadyMessage);
			mediumFont.drawString(
					200-mediumFont.getWidth(
							""+(int)(getReadyTimer/(float)getReadyDelay))/2f, 340, 
							"" + (int)(4-(getReadyTimer/(float)getReadyDelay)*4));
		}
		
		if (gameOver) {
			g.setColor(new Color(0.3f, 0.3f, 0.3f, 0.7f));
			g.fillRoundRect(85, 300, 230, 75, 10);
			mediumFont.drawString(200-mediumFont.getWidth("Game Over")/2f, 305, "Game Over");
			smallFont.drawString(200-smallFont.getWidth("SCORE " + String.format("%010d", score))/2f, 340, "SCORE " + String.format("%010d", score));
		}
	}
}
