package org.bossattack.speedblazer;

import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.fills.GradientFill;
import org.newdawn.slick.geom.Rectangle;
import org.newdawn.slick.state.BasicGameState;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;

public class TitleState extends BasicGameState {

	StateBasedGame stateBasedGame;
	Game game;
	
	Rectangle bgShape = new Rectangle(0, 0, 400, 600);
	GradientFill bgFill = new GradientFill(200,300,Color.black,200,1200,new Color(132,0,255));
	
	String startMessage = "PRESS SPACE TO START";
	float fade = 0;
	int fadeTimer, fadeDelay = 500;
	boolean fadeUp;
	
	int previousScore = 0;
	int bestScore = 0;
	
	public int getID() {
		return 0;
	}
	
	public void setScore(int score) {
		previousScore = score;
		if (score > bestScore) {
			bestScore = score;
		}
	}

	public void init(GameContainer container, StateBasedGame stateBasedGame) throws SlickException {
		game = (Game)(stateBasedGame.getState(1));
		this.stateBasedGame = stateBasedGame;
	}
	
	public void update(GameContainer container, StateBasedGame stateBasedGame, int delta) throws SlickException {
		
		if (fadeTimer < fadeDelay) {
			fadeTimer += delta;
		}
		else {
			fadeTimer = 0;
			fadeUp = !fadeUp;
		}
		
		if (fadeUp) {
			fade = fadeTimer/(float)fadeDelay;
		}
		else {
			fade = (fadeDelay-fadeTimer)/(float)fadeDelay;
		}
	}
	
	public void keyPressed(int key, char c) {
		if (key == Input.KEY_ESCAPE) {
			System.exit(0);
		}
		
		if (key == Input.KEY_SPACE) {
			game.reset();
			stateBasedGame.enterState(1, new FadeOutTransition(Color.black, 250),
					  					 new FadeInTransition(Color.black, 250));
		}
	}
	
	public void render(GameContainer container, StateBasedGame stateBasedGame, Graphics g) throws SlickException {
		g.fill(bgShape, bgFill);
		
		game.largeFont.drawString(60, 80, "SPEED");
		game.largeFont.drawString(120, 140, "BLAZER");
		
		game.smallFont.drawString(200 - game.smallFont.getWidth(startMessage)/2f, 450, startMessage, new Color(1f, 1f, 1f, fade));
		
		game.smallFont.drawString(10, 550, "PREVIOUS SCORE: ");
		game.smallFont.drawString(250, 550, "BEST SCORE: ");
		
		game.smallFont.drawString(10, 570, String.format("%010d", previousScore));
		game.smallFont.drawString(250, 570, String.format("%010d", bestScore));
		
	}
}
